@@ -30,11 +30,19 @@
 #include <ctype.h>
 #include <math.h>
 #include <err.h>
+#include <wordexp.h>
+
+#define alisp_seq_iterator alisp_object
 
 #include "local.h"
 #include "alisp.h"
 #include "alisp_local.h"
 
+
+#define ALISP_FREE_OBJ_POOL	500	/* free objects above this pool */
+#define ALISP_AUTO_GC_THRESHOLD	200	/* run automagically garbage-collect when this threshold is reached */
+#define ALISP_MAIN_ID		"---alisp---main---"
+
 struct alisp_object alsa_lisp_nil;
 struct alisp_object alsa_lisp_t;
 
@@ -48,6 +56,9 @@ static struct alisp_object * eval(struct alisp_instance *instance, struct alisp_
 static struct alisp_object *F_eval(struct alisp_instance *instance, struct alisp_object *);
 static struct alisp_object *F_progn(struct alisp_instance *instance, struct alisp_object *);
 
+/* others */
+static int alisp_include_file(struct alisp_instance *instance, const char *filename);
+
 /*
  *  object handling
  */
@@ -119,6 +130,7 @@ static struct alisp_object * new_object(struct alisp_instance *instance, int typ
 			nomem();
 			return NULL;
 		}
+		++instance->gc_thr_objs;
 		lisp_debug(instance, "allocating cons %p", p);
 	} else {
 		p = instance->free_objs_list;
@@ -138,6 +150,8 @@ static struct alisp_object * new_object(struct alisp_instance *instance, int typ
 	p->gc = 1;
 
 	++instance->used_objs;
+	if (instance->used_objs + instance->free_objs > instance->max_objs)
+		instance->max_objs = instance->used_objs + instance->free_objs;
 
 	return p;
 }
@@ -171,60 +185,6 @@ static void free_objects(struct alisp_instance *instance)
 	}
 }
 
-static struct alisp_object * new_integer(struct alisp_instance *instance, long value)
-{
-	struct alisp_object * obj;
-	
-	obj = new_object(instance, ALISP_OBJ_INTEGER);
-	if (obj)
-		obj->value.i = value;
-	return obj;
-}
-
-static struct alisp_object * new_float(struct alisp_instance *instance, double value)
-{
-	struct alisp_object * obj;
-	
-	obj = new_object(instance, ALISP_OBJ_FLOAT);
-	if (obj)
-		obj->value.f = value;
-	return obj;
-}
-
-static struct alisp_object * new_string(struct alisp_instance *instance, const char *str)
-{
-	struct alisp_object * obj;
-	
-	obj = new_object(instance, ALISP_OBJ_STRING);
-	if (obj && (obj->value.s = strdup(str)) == NULL) {
-		nomem();
-		return NULL;
-	}
-	return obj;
-}
-
-static struct alisp_object * new_identifier(struct alisp_instance *instance, const char *id)
-{
-	struct alisp_object * obj;
-	
-	obj = new_object(instance, ALISP_OBJ_IDENTIFIER);
-	if (obj && (obj->value.id = strdup(id)) == NULL) {
-		nomem();
-		return NULL;
-	}
-	return obj;
-}
-
-static struct alisp_object * new_pointer(struct alisp_instance *instance, const void *ptr)
-{
-	struct alisp_object * obj;
-	
-	obj = new_object(instance, ALISP_OBJ_POINTER);
-	if (obj)
-		obj->value.ptr = ptr;
-	return obj;
-}
-
 static struct alisp_object * search_object_identifier(struct alisp_instance *instance, const char *s)
 {
 	struct alisp_object * p;
@@ -269,6 +229,104 @@ static struct alisp_object * search_object_float(struct alisp_instance *instance
 	return NULL;
 }
 
+static struct alisp_object * search_object_pointer(struct alisp_instance *instance, const void *ptr)
+{
+	struct alisp_object * p;
+
+	for (p = instance->used_objs_list; p != NULL; p = p->next)
+		if (p->type == ALISP_OBJ_POINTER && p->value.ptr == ptr)
+			return p;
+
+	return NULL;
+}
+
+static struct alisp_object * new_integer(struct alisp_instance *instance, long value)
+{
+	struct alisp_object * obj;
+	
+	obj = search_object_integer(instance, value);
+	if (obj != NULL)
+		return obj;
+	obj = new_object(instance, ALISP_OBJ_INTEGER);
+	if (obj)
+		obj->value.i = value;
+	return obj;
+}
+
+static struct alisp_object * new_float(struct alisp_instance *instance, double value)
+{
+	struct alisp_object * obj;
+	
+	obj = search_object_float(instance, value);
+	if (obj != NULL)
+		return obj;
+	obj = new_object(instance, ALISP_OBJ_FLOAT);
+	if (obj)
+		obj->value.f = value;
+	return obj;
+}
+
+static struct alisp_object * new_string(struct alisp_instance *instance, const char *str)
+{
+	struct alisp_object * obj;
+	
+	obj = search_object_string(instance, str);
+	if (obj != NULL)
+		return obj;
+	obj = new_object(instance, ALISP_OBJ_STRING);
+	if (obj && (obj->value.s = strdup(str)) == NULL) {
+		nomem();
+		return NULL;
+	}
+	return obj;
+}
+
+static struct alisp_object * new_identifier(struct alisp_instance *instance, const char *id)
+{
+	struct alisp_object * obj;
+	
+	obj = search_object_identifier(instance, id);
+	if (obj != NULL)
+		return obj;
+	obj = new_object(instance, ALISP_OBJ_IDENTIFIER);
+	if (obj && (obj->value.id = strdup(id)) == NULL) {
+		nomem();
+		return NULL;
+	}
+	return obj;
+}
+
+static struct alisp_object * new_pointer(struct alisp_instance *instance, const void *ptr)
+{
+	struct alisp_object * obj;
+	
+	obj = search_object_pointer(instance, ptr);
+	if (obj != NULL)
+		return obj;
+	obj = new_object(instance, ALISP_OBJ_POINTER);
+	if (obj)
+		obj->value.ptr = ptr;
+	return obj;
+}
+
+static struct alisp_object * new_cons_pointer(struct alisp_instance * instance, const char *ptr_id, void *ptr)
+{
+	struct alisp_object * lexpr;
+
+	if (ptr == NULL)
+		return &alsa_lisp_nil;
+	lexpr = new_object(instance, ALISP_OBJ_CONS);
+	if (lexpr == NULL)
+		return NULL;
+	lexpr->value.c.car = new_string(instance, ptr_id);
+	if (lexpr->value.c.car == NULL)
+		return NULL;
+	lexpr->value.c.cdr = new_pointer(instance, ptr);
+	if (lexpr->value.c.cdr == NULL)
+		return NULL;
+	return lexpr;
+}
+
 void alsa_lisp_init_objects(void) __attribute__ ((constructor));
 
 void alsa_lisp_init_objects(void)
@@ -523,10 +581,13 @@ static struct alisp_object * parse_form(struct alisp_instance *instance)
 		return first;
 }
 
-static struct alisp_object * parse_quote(struct alisp_instance *instance)
+static struct alisp_object * quote_object(struct alisp_instance *instance, struct alisp_object * obj)
 {
 	struct alisp_object * p;
 
+	if (obj == NULL)
+		return NULL;
+
 	p = new_object(instance, ALISP_OBJ_CONS);
 	if (p == NULL)
 		return NULL;
@@ -537,13 +598,16 @@ static struct alisp_object * parse_quote(struct alisp_instance *instance)
 	p->value.c.cdr = new_object(instance, ALISP_OBJ_CONS);
 	if (p->value.c.cdr == NULL)
 		return NULL;
-	p->value.c.cdr->value.c.car = parse_object(instance, 0);
-	if (p->value.c.cdr->value.c.car == NULL)
-		return NULL;
 
+	p->value.c.cdr->value.c.car = obj;
 	return p;
 }
 
+static inline struct alisp_object * parse_quote(struct alisp_instance *instance)
+{
+	return quote_object(instance, parse_object(instance, 0));
+}
+
 static struct alisp_object * parse_object(struct alisp_instance *instance, int havetoken)
 {
 	int thistoken;
@@ -569,28 +633,20 @@ static struct alisp_object * parse_object(struct alisp_instance *instance, int h
 		else if (!strcmp(instance->token_buffer, "nil"))
 			p = &alsa_lisp_nil;
 		else {
-			if ((p = search_object_identifier(instance, instance->token_buffer)) == NULL)
-				p = new_identifier(instance, instance->token_buffer);
+			p = new_identifier(instance, instance->token_buffer);
 		}
 		break;
 	case ALISP_INTEGER: {
-		long i;
-		i = atol(instance->token_buffer);
-		if ((p = search_object_integer(instance, i)) == NULL)
-			p = new_integer(instance, i);
+		p = new_integer(instance, atol(instance->token_buffer));
 		break;
 	}
 	case ALISP_FLOAT:
 	case ALISP_FLOATE: {
-		double f;
-		f = atof(instance->token_buffer);
-		if ((p = search_object_float(instance, f)) == NULL)
-			p = new_float(instance, f);
+		p = new_float(instance, atof(instance->token_buffer));
 		break;
 	}
 	case ALISP_STRING:
-		if ((p = search_object_string(instance, instance->token_buffer)) == NULL)
-			p = new_string(instance, instance->token_buffer);
+		p = new_string(instance, instance->token_buffer);
 		break;
 	default:
 		lisp_warn(instance, "%d:%d: unexpected character `%c'", instance->lineno, instance->charno, thistoken);
@@ -604,44 +660,76 @@ static struct alisp_object * parse_object(struct alisp_instance *instance, int h
  *  object manipulation
  */
 
-static int set_object(struct alisp_instance *instance, struct alisp_object * name, struct alisp_object * value)
+static struct alisp_object_pair * set_object(struct alisp_instance *instance, struct alisp_object * name, struct alisp_object * value)
 {
 	struct alisp_object_pair *p;
 
 	if (name->value.id == NULL)
-		return 0;
+		return NULL;
 
 	for (p = instance->setobjs_list; p != NULL; p = p->next)
 		if (p->name->value.id != NULL &&
 		    !strcmp(name->value.id, p->name->value.id)) {
 			p->value = value;
-			return 0;
+			return p;
 		}
 
 	p = (struct alisp_object_pair *)malloc(sizeof(struct alisp_object_pair));
 	if (p == NULL) {
 		nomem();
-		return -ENOMEM;
+		return NULL;
 	}
 	p->next = instance->setobjs_list;
 	instance->setobjs_list = p;
 	p->name = name;
 	p->value = value;
-	return 0;
+	return p;
 }
 
-static struct alisp_object * get_object(struct alisp_instance *instance, struct alisp_object * name)
+static struct alisp_object * unset_object1(struct alisp_instance *instance, const char *id)
+{
+	struct alisp_object * res;
+	struct alisp_object_pair *p, *p1;
+
+	for (p = instance->setobjs_list, p1 = NULL; p != NULL; p1 = p, p = p->next) {
+		if (p->name->value.id != NULL &&
+		    !strcmp(id, p->name->value.id)) {
+		    	if (p1)
+		    		p1->next = p->next;
+		    	else
+		    		instance->setobjs_list = p->next;
+		    	res = p->value;
+		    	free(p);
+		    	return res;
+		}
+	}
+	
+	return &alsa_lisp_nil;
+}
+
+static inline struct alisp_object * unset_object(struct alisp_instance *instance, struct alisp_object * name)
+{
+	return unset_object1(instance, name->value.id);
+}
+
+static struct alisp_object * get_object1(struct alisp_instance *instance, const char *id)
 {
 	struct alisp_object_pair *p;
 
-	for (p = instance->setobjs_list; p != NULL; p = p->next)
+	for (p = instance->setobjs_list; p != NULL; p = p->next) {
 		if (p->name->value.id != NULL &&
-		    !strcmp(name->value.id, p->name->value.id))
+		    !strcmp(id, p->name->value.id))
 			return p->value;
+	}
 
 	return &alsa_lisp_nil;
 }
 
+static inline struct alisp_object * get_object(struct alisp_instance *instance, struct alisp_object * name)
+{
+	return get_object1(instance, name->value.id);
+}
+
 static void dump_objects(struct alisp_instance *instance, const char *fname)
 {
 	struct alisp_object_pair *p;
@@ -666,6 +754,8 @@ static void dump_objects(struct alisp_instance *instance, const char *fname)
 		    	snd_output_printf(out, ")\n");
 		    	continue;
 		}
+		if (!strcmp(p->name->value.id, ALISP_MAIN_ID))	/* internal thing */
+			continue;
 		snd_output_printf(out, "(setq %s '", p->name->value.id);
  		princ_object(out, p->value);
 		snd_output_printf(out, ")\n");
@@ -683,6 +773,7 @@ static const char *obj_type_str(struct alisp_object * p)
 	case ALISP_OBJ_FLOAT: return "float";
 	case ALISP_OBJ_IDENTIFIER: return "identifier";
 	case ALISP_OBJ_STRING: return "string";
+	case ALISP_OBJ_POINTER: return "pointer";
 	case ALISP_OBJ_CONS: return "cons";
 	default: assert(0);
 	}
@@ -777,10 +868,16 @@ static void do_garbage_collect(struct alisp_instance *instance)
 			lisp_debug(instance, "** collecting cons %p", p);
 			free_object(p);
 
-			p->next = instance->free_objs_list;
-			instance->free_objs_list = p;
+			if (instance->free_objs < ALISP_FREE_OBJ_POOL) {
+				p->next = instance->free_objs_list;
+				instance->free_objs_list = p;
+				++instance->free_objs;
+				if (instance->gc_thr_objs > 0)
+					instance->gc_thr_objs--;
+			} else {
+				free(p);
+			}
 
-			++instance->free_objs;
 			--instance->used_objs;
 		} else {
 			/* The object is referenced somewhere. */
@@ -792,9 +889,17 @@ static void do_garbage_collect(struct alisp_instance *instance)
 	instance->used_objs_list = new_used_objs_list;
 }
 
+static inline void auto_garbage_collect(struct alisp_instance *instance)
+{
+	if (instance->gc_thr_objs >= ALISP_AUTO_GC_THRESHOLD) {
+		do_garbage_collect(instance);
+		instance->gc_thr_objs = 0;
+	}
+}
+
 static void garbage_collect(struct alisp_instance *instance)
 {
-	if (++instance->gc_id == INT_MAX)
+	if (++instance->gc_id == 255)
 		instance->gc_id = 1;
 	do_garbage_collect(instance);
 }
@@ -851,33 +956,64 @@ static struct alisp_object * F_cdr(struct alisp_instance *instance, struct alisp
 static struct alisp_object * F_add(struct alisp_instance *instance, struct alisp_object * args)
 {
 	struct alisp_object * p = args, * p1;
-	long v = 0;
-	double f = 0;
-	int type = ALISP_OBJ_INTEGER;
 
-	do {
-		p1 = eval(instance, car(p));
-		if (p1->type == ALISP_OBJ_INTEGER) {
-			if (type == ALISP_OBJ_FLOAT)
-				f += p1->value.i;
-			else
-				v += p1->value.i;
-		} else if (p1->type == ALISP_OBJ_FLOAT) {
-			f += p1->value.f + v;
-			v = 0;
-			type = ALISP_OBJ_FLOAT;
-		} else {
-			lisp_warn(instance, "sum with a non integer or float operand");
+	p1 = eval(instance, car(p));
+	if (p1->type == ALISP_OBJ_INTEGER || p1->type == ALISP_OBJ_FLOAT) {
+		long v = 0;
+		double f = 0;
+		int type = ALISP_OBJ_INTEGER;
+		for (;;) {
+			if (p1->type == ALISP_OBJ_INTEGER) {
+				if (type == ALISP_OBJ_FLOAT)
+					f += p1->value.i;
+				else
+					v += p1->value.i;
+			} else if (p1->type == ALISP_OBJ_FLOAT) {
+				f += p1->value.f + v;
+				v = 0;
+				type = ALISP_OBJ_FLOAT;
+			} else {
+				lisp_warn(instance, "sum with a non integer or float operand");
+			}
+			p = cdr(p);
+			if (p == &alsa_lisp_nil)
+				break;
+			p1 = eval(instance, car(p));
 		}
-		p = cdr(p);
-	} while (p != &alsa_lisp_nil);
-
-	if (type == ALISP_OBJ_INTEGER) {
-		p1 = new_integer(instance, v);
-	} else {
-		p1 = new_float(instance, f);
+		if (type == ALISP_OBJ_INTEGER) {
+			return new_integer(instance, v);
+		} else {
+			return new_float(instance, f);
+		}
+	} else if (p1->type == ALISP_OBJ_STRING) {
+		char *str = NULL, *str1;
+		for (;;) {
+			if (p1->type == ALISP_OBJ_STRING) {
+				str1 = realloc(str, (str ? strlen(str) : 0) + strlen(p1->value.s) + 1);
+				if (str1 == NULL) {
+					nomem();
+					if (str)
+						free(str);
+					return NULL;
+				}
+				if (str == NULL)
+					strcpy(str1, p1->value.s);
+				else
+					strcat(str1, p1->value.s);
+				str = str1;
+			} else {
+				lisp_warn(instance, "concat with a non string or identifier operand");
+			}
+			p = cdr(p);
+			if (p == &alsa_lisp_nil)
+				break;
+			p1 = eval(instance, car(p));
+		}
+		p = new_string(instance, str);
+		free(str);
+		return p;
 	}
-	return p1;
+	return &alsa_lisp_nil;
 }
 
 /*
@@ -917,11 +1053,10 @@ static struct alisp_object * F_sub(struct alisp_instance *instance, struct alisp
 	} while (p != &alsa_lisp_nil);
 
 	if (type == ALISP_OBJ_INTEGER) {
-		p1 = new_integer(instance, v);
+		return new_integer(instance, v);
 	} else {
-		p1 = new_object(instance, f);
+		return new_object(instance, f);
 	}
-	return p1;
 }
 
 /*
@@ -951,12 +1086,10 @@ static struct alisp_object * F_mul(struct alisp_instance *instance, struct alisp
 	} while (p != &alsa_lisp_nil);
 
 	if (type == ALISP_OBJ_INTEGER) {
-		p1 = new_integer(instance, v);
+		return new_integer(instance, v);
 	} else {
-		p1 = new_float(instance, f);
+		return new_float(instance, f);
 	}
-
-	return p1;
 }
 
 /*
@@ -1009,12 +1142,10 @@ static struct alisp_object * F_div(struct alisp_instance *instance, struct alisp
 	} while (p != &alsa_lisp_nil);
 
 	if (type == ALISP_OBJ_INTEGER) {
-		p1 = new_integer(instance, v);
+		return new_integer(instance, v);
 	} else {
-		p1 = new_float(instance, f);
+		return new_float(instance, f);
 	}
-
-	return p1;
 }
 
 /*
@@ -1194,6 +1325,27 @@ static struct alisp_object * F_numeq(struct alisp_instance *instance, struct ali
 	return &alsa_lisp_nil;
 }
 
+/*
+ * Syntax: (exfun name)
+ * Test, if a function exists
+ */
+static struct alisp_object * F_exfun(struct alisp_instance *instance, struct alisp_object * args)
+{
+	struct alisp_object * p1, * p2;
+
+	p1 = car(args);
+	if (p1->type != ALISP_OBJ_STRING && p1->type != ALISP_OBJ_IDENTIFIER)
+		return &alsa_lisp_nil;
+	p2 = get_object(instance, p1);
+	if (p2 == &alsa_lisp_nil)
+		return &alsa_lisp_nil;
+	p2 = car(p2);
+	if (p2->type == ALISP_OBJ_IDENTIFIER && !strcmp(p2->value.id, "lambda"))
+		return &alsa_lisp_t;
+
+	return &alsa_lisp_nil;
+}
+
 static void princ_string(snd_output_t *out, char *s)
 {
 	char *p;
@@ -1350,31 +1502,27 @@ static inline int eq(struct alisp_object * p1, struct alisp_object * p2)
 
 static int equal(struct alisp_object * p1, struct alisp_object * p2)
 {
-	if (eq(p1, p1))
+	int type1, type2;
+
+	if (eq(p1, p2))
 		return 1;
 
-	if (p1->type == ALISP_OBJ_CONS || p2->type == ALISP_OBJ_CONS)
+	type1 = p1->type;
+	type2 = p2->type;
+
+	if (type1 == ALISP_OBJ_CONS || type2 == ALISP_OBJ_CONS)
 		return 0;
 
-	if (p1->type == p2->type)
-		switch (p1->type) {
-		case ALISP_OBJ_IDENTIFIER:
-			if (!strcmp(p1->value.id, p2->value.id))
-				return 1;
-			return 0;
+	if (type1 == type2) {
+		switch (type1) {
 		case ALISP_OBJ_STRING:
-			if (!strcmp(p1->value.s, p2->value.s))
-				return 1;
-			return 0;
+			return !strcmp(p1->value.s, p2->value.s);
 		case ALISP_OBJ_INTEGER:
-			if (p1->value.i == p2->value.i)
-				return 1;
-			return 0;
+			return p1->value.i == p2->value.i;
 		case ALISP_OBJ_FLOAT:
-			if (p1->value.i == p2->value.i)
-				return 1;
-			return 0;
+			return p1->value.i == p2->value.i;
 		}
+	}
 
 	return 0;
 }
@@ -1615,12 +1763,23 @@ static struct alisp_object * F_set(struct alisp_instance *instance, struct alisp
 	if (p1 == &alsa_lisp_nil) {
 		lisp_warn(instance, "setting the value of a nil object");
 	} else
-		if (set_object(instance, p1, p2))
+		if (set_object(instance, p1, p2) == NULL)
 			return NULL;
 
 	return p2;
 }
 
+/*
+ * Syntax: (unset name)
+ */
+static struct alisp_object * F_unset(struct alisp_instance *instance, struct alisp_object * args)
+{
+	struct alisp_object * p1 = eval(instance, car(args));
+
+	unset_object(instance, p1);
+	return p1;
+}
+
 /*
  * Syntax: (setq name value...)
  * Syntax: (setf name value...)
@@ -1633,7 +1792,7 @@ static struct alisp_object * F_setq(struct alisp_instance *instance, struct alis
 	do {
 		p1 = car(p);
 		p2 = eval(instance, car(cdr(p)));
-		if (set_object(instance, p1, p2))
+		if (set_object(instance, p1, p2) == NULL)
 			return NULL;
 		p = cdr(cdr(p));
 	} while (p != &alsa_lisp_nil);
@@ -1641,6 +1800,24 @@ static struct alisp_object * F_setq(struct alisp_instance *instance, struct alis
 	return p2;
 }
 
+/*
+ * Syntax: (unsetq name...)
+ * Syntax: (unsetf name...)
+ * `name' is not evalled
+ */
+static struct alisp_object * F_unsetq(struct alisp_instance *instance, struct alisp_object * args)
+{
+	struct alisp_object * p = args, * p1, * res;
+
+	do {
+		p1 = car(p);
+		res = unset_object(instance, p1);
+		p = cdr(p);
+	} while (p != &alsa_lisp_nil);
+
+	return res;
+}
+
 /*
  * Syntax: (defun name arglist expr...)
  * `name' is not evalled
@@ -1661,7 +1838,7 @@ static struct alisp_object * F_defun(struct alisp_instance *instance, struct ali
 		lexpr->value.c.cdr->value.c.car = p2;
 		lexpr->value.c.cdr->value.c.cdr = p3;
 
-		if (set_object(instance, p1, lexpr))
+		if (set_object(instance, p1, lexpr) == NULL)
 			return NULL;
 	}
 
@@ -1671,7 +1848,7 @@ static struct alisp_object * F_defun(struct alisp_instance *instance, struct ali
 static struct alisp_object * eval_func(struct alisp_instance *instance, struct alisp_object * p, struct alisp_object * args)
 {
 	struct alisp_object * p1, * p2, * p3, * p4, * p5;
-	struct alisp_object * eval_objs[64], * save_objs[64];
+	struct alisp_object ** eval_objs, ** save_objs;
 	int i;
 
 	p1 = car(p);
@@ -1679,33 +1856,40 @@ static struct alisp_object * eval_func(struct alisp_instance *instance, struct a
 		p2 = car(cdr(p));
 		p3 = args;
 
-		if (count_list(p2) != count_list(p3)) {
+		if ((i = count_list(p2)) != count_list(p3)) {
 			lisp_warn(instance, "wrong number of parameters");
 			return &alsa_lisp_nil;
 		}
 
+		eval_objs = malloc(2 * i * sizeof(struct alisp_object *));
+		if (eval_objs == NULL) {
+			nomem();
+			goto _err;
+		}
+		save_objs = eval_objs + i;
+		
 		/*
 		 * Save the new variable values.
 		 */
 		i = 0;
-		do {
+		while (p3 != &alsa_lisp_nil) {
 			p5 = eval(instance, car(p3));
 			eval_objs[i++] = p5;
 			p3 = cdr(p3);
-		} while (p3 != &alsa_lisp_nil);
+		}
 
 		/*
 		 * Save the old variable values and set the new ones.
 		 */
 		i = 0;
-		do {
+		while (p2 != &alsa_lisp_nil) {
 			p4 = car(p2);
 			save_objs[i] = get_object(instance, p4);
-			if (set_object(instance, p4, eval_objs[i]))
-				return NULL;
+			if (set_object(instance, p4, eval_objs[i]) == NULL)
+				goto _err;
 			p2 = cdr(p2);
 			++i;
-		} while (p2 != &alsa_lisp_nil);
+		}
 
 		p5 = F_progn(instance, cdr(cdr(p)));
 
@@ -1714,17 +1898,25 @@ static struct alisp_object * eval_func(struct alisp_instance *instance, struct a
 		 */
 		p2 = car(cdr(p));
 		i = 0;
-		do {
+		while (p2 != &alsa_lisp_nil) {
 			p4 = car(p2);
-			if (set_object(instance, p4, save_objs[i++]))
+			if (set_object(instance, p4, save_objs[i++]) == NULL)
 				return NULL;
 			p2 = cdr(p2);
-		} while (p2 != &alsa_lisp_nil);
+		}
+
+		if (eval_objs)
+			free(eval_objs);
 
 		return p5;
 	}
 
 	return &alsa_lisp_nil;
+
+       _err:
+	if (eval_objs)
+		free(eval_objs);
+       	return NULL;
 }
 
 struct alisp_object * F_gc(struct alisp_instance *instance, struct alisp_object * args ATTRIBUTE_UNUSED)
@@ -1734,6 +1926,40 @@ struct alisp_object * F_gc(struct alisp_instance *instance, struct alisp_object
 	return &alsa_lisp_t;
 }
 
+/*
+ * Syntax: (path what)
+ * what is string ('data')
+ */
+struct alisp_object * F_path(struct alisp_instance *instance, struct alisp_object * args)
+{
+	struct alisp_object * p = args, * p1;
+
+	p1 = eval(instance, car(p));
+	if (p1->type != ALISP_OBJ_STRING)
+		return &alsa_lisp_nil;
+	if (!strcmp(p1->value.s, "data"))
+		return new_string(instance, DATADIR);
+	return &alsa_lisp_nil;
+}
+
+/*
+ * Syntax: (include filename...)
+ */
+struct alisp_object * F_include(struct alisp_instance *instance, struct alisp_object * args)
+{
+	struct alisp_object * p = args, * p1;
+	int res = -ENOENT;
+
+	do {
+		p1 = eval(instance, car(p));
+		if (p1->type == ALISP_OBJ_STRING)
+			res = alisp_include_file(instance, p1->value.s);
+		p = cdr(p);
+	} while (p != &alsa_lisp_nil);
+
+	return new_integer(instance, res);
+}
+
 /*
  * Syntax: (int value)
  * 'value' can be integer or float type
@@ -1849,6 +2075,27 @@ struct alisp_object * F_assq(struct alisp_instance *instance, struct alisp_objec
 	return &alsa_lisp_nil;	
 }
 
+/*
+ *  Syntax: (nth index alist)
+ */
+struct alisp_object * F_nth(struct alisp_instance *instance, struct alisp_object * args)
+{
+	struct alisp_object * p1, * p2;
+	long idx;
+
+	p1 = eval(instance, car(args));
+	p2 = eval(instance, car(cdr(args)));
+
+	if (p1->type != ALISP_OBJ_INTEGER)
+		return &alsa_lisp_nil;
+	if (p2->type != ALISP_OBJ_CONS)
+		return &alsa_lisp_nil;
+	idx = p1->value.i;
+	while (idx-- > 0)
+		p2 = cdr(p2);
+	return car(p2);
+}
+
 /*
  *  Syntax: (rassq value alist)
  */
@@ -1884,6 +2131,19 @@ static struct alisp_object * F_dump_memory(struct alisp_instance *instance, stru
 	return &alsa_lisp_nil;
 }
 
+static struct alisp_object * F_stat_memory(struct alisp_instance *instance, struct alisp_object * args ATTRIBUTE_UNUSED)
+{
+	snd_output_printf(instance->out, "*** Memory stats\n");
+	snd_output_printf(instance->out, "  used_objs = %li, free_objs = %li, max_objs = %li, obj_size = %i (total bytes = %li, max bytes = %li)\n",
+		instance->used_objs,
+		instance->free_objs,
+		instance->max_objs,
+		sizeof(struct alisp_object),
+		(instance->used_objs + instance->free_objs) * sizeof(struct alisp_object),
+		instance->max_objs * sizeof(struct alisp_object));
+	return &alsa_lisp_nil;
+}
+
 static struct alisp_object * F_dump_objects(struct alisp_instance *instance, struct alisp_object * args)
 {
 	struct alisp_object * p = car(args);
@@ -1909,6 +2169,7 @@ static struct intrinsic intrinsics[] = {
 	{ "%", F_mod },
 	{ "&dump-memory", F_dump_memory },
 	{ "&dump-objects", F_dump_objects },
+	{ "&stat-memory", F_stat_memory },
 	{ "*", F_mul },
 	{ "+", F_add },
 	{ "-", F_sub },
@@ -1930,15 +2191,19 @@ static struct intrinsic intrinsics[] = {
 	{ "eq", F_eq },
 	{ "equal", F_equal },
 	{ "eval", F_eval },
+	{ "exfun", F_exfun },
 	{ "float", F_float },
 	{ "garbage-collect", F_gc },
 	{ "gc", F_gc },
 	{ "if", F_if },
+	{ "include", F_include },
 	{ "int", F_int },
 	{ "list", F_list },
 	{ "not", F_not },
+	{ "nth", F_nth },
 	{ "null", F_not },
 	{ "or", F_or },
+	{ "path", F_path },
 	{ "princ", F_princ },
 	{ "prog1", F_prog1 },
 	{ "prog2", F_prog2 },
@@ -1950,7 +2215,12 @@ static struct intrinsic intrinsics[] = {
 	{ "setf", F_setq },
 	{ "setq", F_setq },
 	{ "str", F_str },
+	{ "string=", F_equal },
+	{ "string-equal", F_equal },
 	{ "unless", F_unless },
+	{ "unset", F_unset },
+	{ "unsetf", F_unsetq },
+	{ "unsetq", F_unsetq },
 	{ "when", F_when },
 	{ "while", F_while },
 };
@@ -1972,6 +2242,9 @@ static struct alisp_object * eval_cons(struct alisp_instance *instance, struct a
 
 		if (!strcmp(p1->value.id, "lambda"))
 			return p;
+
+		auto_garbage_collect(instance);
+
 		key.name = p1->value.id;
 		if ((item = bsearch(&key, intrinsics,
 				    sizeof intrinsics / sizeof intrinsics[0],
@@ -1998,7 +2271,9 @@ static struct alisp_object * eval(struct alisp_instance *instance, struct alisp_
 	case ALISP_OBJ_IDENTIFIER:
 		return get_object(instance, p);
 	case ALISP_OBJ_INTEGER:
+	case ALISP_OBJ_FLOAT:
 	case ALISP_OBJ_STRING:
+	case ALISP_OBJ_POINTER:
 		return p;
 	case ALISP_OBJ_CONS:
 		return eval_cons(instance, p);
@@ -2015,17 +2290,98 @@ static struct alisp_object * F_eval(struct alisp_instance *instance, struct alis
 /*
  *  main routine
  */
+
+static int alisp_include_file(struct alisp_instance *instance, const char *filename)
+{
+	snd_input_t *old_in;
+	struct alisp_object *p, *p1, *omain;
+	struct alisp_object_pair *pmain;
+	char *name, *uname;
+	int retval = 0, err;
+
+	err = snd_user_file(filename, &name);
+	if (err < 0)
+		return err;
+	old_in = instance->in;
+	err = snd_input_stdio_open(&instance->in, name, "r");
+	if (err < 0) {
+		retval = err;
+		goto _err;
+	}
+	if (instance->verbose)
+		lisp_verbose(instance, "** include filename '%s'", name);
+	uname = malloc(sizeof(ALISP_MAIN_ID) + strlen(name) + 2);
+	if (uname == NULL) {
+		retval = -ENOMEM;
+		goto _err;
+	}
+	strcpy(uname, ALISP_MAIN_ID);
+	strcat(uname, "-");
+	strcat(uname, name);
+	omain = new_identifier(instance, uname);
+	free(uname);
+	if (omain == NULL) {
+		retval = -ENOMEM;
+		goto _err;
+	}
+	pmain = set_object(instance, omain, &alsa_lisp_t);
+	if (pmain == NULL) {
+		retval = -ENOMEM;
+		goto _err;
+	}
+
+	for (;;) {
+		if ((p = parse_object(instance, 0)) == NULL)
+			break;
+		if (instance->verbose) {
+			lisp_verbose(instance, "** code");
+			princ_object(instance->vout, p);
+			snd_output_putc(instance->vout, '\n');
+		}
+		pmain->value = p;		/* protect the code tree from garbage-collect */
+		p1 = eval(instance, p);
+		if (p1 == NULL) {
+			retval = -ENOMEM;
+			break;
+		}
+		if (instance->verbose) {
+			lisp_verbose(instance, "** result");
+			princ_object(instance->vout, p1);
+			snd_output_putc(instance->vout, '\n');
+		}
+		if (instance->debug) {
+			lisp_debug(instance, "** objects before collection");
+			print_obj_lists(instance, instance->dout);
+		}
+		pmain->value = &alsa_lisp_t;	/* let garbage-collect working */
+		garbage_collect(instance);
+		if (instance->debug) {
+			lisp_debug(instance, "** objects after collection");
+			print_obj_lists(instance, instance->dout);
+		}
+	}	
+
+	unset_object(instance, omain);
+
+       _err:
+	free(name);
+	instance->in = old_in;
+	return retval;
+}
  
 int alsa_lisp(struct alisp_cfg *cfg, struct alisp_instance **_instance)
 {
 	struct alisp_instance *instance;
-	struct alisp_object *p, *p1;
+	struct alisp_object *p, *p1, *omain;
+	struct alisp_object_pair *pmain;
+	int retval = 0;
 	
 	instance = (struct alisp_instance *)malloc(sizeof(struct alisp_instance));
 	if (instance == NULL) {
 		nomem();
 		return -ENOMEM;
 	}
+	memset(instance, 0, sizeof(struct alisp_instance));
 	instance->verbose = cfg->verbose && cfg->vout;
 	instance->warning = cfg->warning && cfg->wout;
 	instance->debug = cfg->debug && cfg->dout;
@@ -2039,6 +2395,17 @@ int alsa_lisp(struct alisp_cfg *cfg, struct alisp_instance **_instance)
 	
 	init_lex(instance);
 
+	omain = new_identifier(instance, ALISP_MAIN_ID);
+	if (omain == NULL) {
+		alsa_lisp_free(instance);
+		return -ENOMEM;
+	}
+	pmain = set_object(instance, omain, &alsa_lisp_t);
+	if (pmain == NULL) {
+		alsa_lisp_free(instance);
+		return -ENOMEM;
+	}
+
 	for (;;) {
 		if ((p = parse_object(instance, 0)) == NULL)
 			break;
@@ -2047,7 +2414,12 @@ int alsa_lisp(struct alisp_cfg *cfg, struct alisp_instance **_instance)
 			princ_object(instance->vout, p);
 			snd_output_putc(instance->vout, '\n');
 		}
+		pmain->value = p;		/* protect the code tree from garbage-collect */
 		p1 = eval(instance, p);
+		if (p1 == NULL) {
+			retval = -ENOMEM;
+			break;
+		}
 		if (instance->verbose) {
 			lisp_verbose(instance, "** result");
 			princ_object(instance->vout, p1);
@@ -2057,6 +2429,7 @@ int alsa_lisp(struct alisp_cfg *cfg, struct alisp_instance **_instance)
 			lisp_debug(instance, "** objects before collection");
 			print_obj_lists(instance, instance->dout);
 		}
+		pmain->value = &alsa_lisp_t;	/* let garbage-collect working */
 		garbage_collect(instance);
 		if (instance->debug) {
 			lisp_debug(instance, "** objects after collection");
@@ -2064,7 +2437,8 @@ int alsa_lisp(struct alisp_cfg *cfg, struct alisp_instance **_instance)
 		}
 	}
 
-	done_lex(instance);
+	unset_object(instance, omain);
+
 	if (_instance)
 		*_instance = instance;
 	else
@@ -2077,6 +2451,223 @@ void alsa_lisp_free(struct alisp_instance *instance)
 {
 	if (instance == NULL)
 		return;
+	done_lex(instance);
 	free_objects(instance);
 	free(instance);
 }
+
+struct alisp_cfg *alsa_lisp_default_cfg(snd_input_t *input)
+{
+	snd_output_t *output, *eoutput;
+	struct alisp_cfg *cfg;
+	int err;
+	
+	err = snd_output_stdio_attach(&output, stdout, 0);
+	if (err < 0)
+		return NULL;
+	err = snd_output_stdio_attach(&eoutput, stderr, 0);
+	if (err < 0) {
+		snd_output_close(output);
+		return NULL;
+	}
+	cfg = calloc(1, sizeof(struct alisp_cfg));
+	if (cfg == NULL) {
+		snd_output_close(eoutput);
+		snd_output_close(output);
+		return NULL;
+	}
+	cfg->out = output;
+	cfg->wout = eoutput;
+	cfg->eout = eoutput;
+	cfg->dout = eoutput;
+	cfg->in = input;
+	return cfg;
+}
+
+void alsa_lisp_default_cfg_free(struct alisp_cfg *cfg)
+{
+	snd_input_close(cfg->in);
+	snd_output_close(cfg->out);
+	snd_output_close(cfg->dout);
+	free(cfg);
+}
+
+int alsa_lisp_function(struct alisp_instance *instance, struct alisp_seq_iterator **result,
+		       const char *id, const char *args, ...)
+{
+	int err = 0;
+	struct alisp_object *aargs = NULL, *obj, *res;
+
+	if (args && *args != 'n') {
+		va_list ap;
+		struct alisp_object *p;
+		p = NULL;
+		va_start(ap, args);
+		while (*args) {
+			if (*args++ != '%') {
+				err = -EINVAL;
+				break;
+			}
+			if (*args == '\0') {
+				err = -EINVAL;
+				break;
+			}
+			obj = NULL;
+			err = 0;
+			switch (*args++) {
+			case 's':
+				obj = new_string(instance, va_arg(ap, char *));
+				break;
+			case 'i':
+				obj = new_integer(instance, va_arg(ap, int));
+				break;
+			case 'l':
+				obj = new_integer(instance, va_arg(ap, long));
+				break;
+			case 'f':
+			case 'd':
+				obj = new_integer(instance, va_arg(ap, double));
+				break;
+			case 'p': {
+				char _ptrid[24];
+				char *ptrid = _ptrid;
+				while (*args && *args != '%')
+					*ptrid++ = *args++;
+				*ptrid = 0;
+				if (ptrid == _ptrid) {
+					err = -EINVAL;
+					break;
+				}
+				obj = new_cons_pointer(instance, _ptrid, va_arg(ap, void *));
+				obj = quote_object(instance, obj);
+				break;
+			}
+			default:
+				err = -EINVAL;
+				break;
+			}
+			if (err < 0)
+				goto __args_end;
+			if (obj == NULL) {
+				err = -ENOMEM;
+				goto __args_end;
+			}
+			if (p == NULL) {
+				p = aargs = new_object(instance, ALISP_OBJ_CONS);
+			} else {
+				p->value.c.cdr = new_object(instance, ALISP_OBJ_CONS);
+				p = p->value.c.cdr;
+			}
+			if (p == NULL) {
+				err = -ENOMEM;
+				goto __args_end;
+			}
+			p->value.c.car = obj;
+		}
+	      __args_end:
+		va_end(ap);
+		if (err < 0)
+			return err;
+#if 0
+		snd_output_printf(instance->wout, ">>>");
+		princ_object(instance->wout, aargs);
+		snd_output_printf(instance->wout, "<<<\n");
+#endif
+	}
+
+	err = -ENOENT;
+	if (aargs == NULL)
+		aargs = &alsa_lisp_nil;
+	if ((obj = get_object1(instance, id)) != &alsa_lisp_nil) {
+		res = eval_func(instance, obj, aargs);
+		err = 0;
+	} else {
+		struct intrinsic key, *item;
+		key.name = id;
+		if ((item = bsearch(&key, intrinsics,
+				    sizeof intrinsics / sizeof intrinsics[0],
+				    sizeof intrinsics[0], compar)) != NULL) {
+			res = item->func(instance, aargs);
+			err = 0;
+		} else if ((item = bsearch(&key, snd_intrinsics,
+				         sizeof snd_intrinsics / sizeof snd_intrinsics[0],
+					 sizeof snd_intrinsics[0], compar)) != NULL) {
+			res = item->func(instance, aargs);
+			err = 0;
+		} else {
+			res = &alsa_lisp_nil;
+		}
+	}
+	if (res == NULL)
+		err = -ENOMEM;
+	if (err == 0 && result)
+		*result = res;
+
+	return 0;
+}
+
+int alsa_lisp_seq_first(struct alisp_instance *instance, const char *id,
+			struct alisp_seq_iterator **seq)
+{
+	struct alisp_object * p1;
+
+	p1 = get_object1(instance, id);
+	if (p1 == NULL)
+		return -ENOMEM;
+	*seq = p1;
+	return 0;
+}
+
+int alsa_lisp_seq_next(struct alisp_seq_iterator **seq)
+{
+	struct alisp_object * p1 = *seq;
+
+	p1 = cdr(p1);
+	if (p1 == &alsa_lisp_nil)
+		return -ENOENT;
+	*seq = p1;
+	return 0;
+}
+
+int alsa_lisp_seq_count(struct alisp_seq_iterator *seq)
+{
+	int count = 0;
+	
+	while (seq != &alsa_lisp_nil) {
+		count++;
+		seq = cdr(seq);
+	}
+	return count;
+}
+
+int alsa_lisp_seq_integer(struct alisp_seq_iterator *seq, long *val)
+{
+	if (seq->type == ALISP_OBJ_CONS)
+		seq = seq->value.c.cdr;
+	if (seq->type == ALISP_OBJ_INTEGER)
+		*val = seq->value.i;
+	else
+		return -EINVAL;
+	return 0;
+}
+
+int alsa_lisp_seq_pointer(struct alisp_seq_iterator *seq, const char *ptr_id, void **ptr)
+{
+	struct alisp_object * p2;
+	
+	if (seq->type == ALISP_OBJ_CONS && seq->value.c.cdr->type == ALISP_OBJ_CONS)
+		seq = seq->value.c.cdr;
+	if (seq->type == ALISP_OBJ_CONS) {
+		p2 = seq->value.c.car;
+		if (p2->type != ALISP_OBJ_STRING)
+			return -EINVAL;
+		if (strcmp(p2->value.s, ptr_id))
+			return -EINVAL;
+		p2 = seq->value.c.cdr;
+		if (p2->type != ALISP_OBJ_POINTER)
+			return -EINVAL;
+		*ptr = (void *)seq->value.ptr;
+	} else
+		return -EINVAL;
+	return 0;
+}
